﻿using System;
using System.IO;
using System.Windows.Forms;
using ABB.Robotics.Controllers;
using ABB.Robotics.Controllers.Discovery;

namespace ExampleControllerBackup
{
    public partial class FormMain : Form
    {
        // Path to the local directory on the computer where the backups will be stored.
        private static string dirBackupPC = "C:\\ControllerBackups";
        // Path to the remote directory on the controller where the backups will be stored relative to /hd0a/XXX-XXXXX/HOME/.
        private static readonly string DIR_BACKUP_CTRL = "TempBackup";

        public FormMain()
        {
            InitializeComponent();
            SetPath(dirBackupPC);
        }

        /// <summary>
        /// 1. Creates a remote backup on the controller.
        /// 2. Copies over the backup to the local computer.
        /// 3. Removes the backup from the controller.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonBackup_Click(object sender, EventArgs e)
        {
            Controller c = null;

            // Create and log on to the controller
            try
            {
                c = ControllerFactory.CreateFrom((ControllerInfo)listView.SelectedItems[0].Tag);
                c.Logon(UserInfo.DefaultUser);
            }
            catch (Exception)
            {
                MessageBox.Show("Could not logon to controller.");
                return;
            }

            // Start the backup operation on the controller
            try
            {
                c.FileSystem.RemoteDirectory = "(HOME)$";
                c.Backup(DIR_BACKUP_CTRL);
                // Wait for the backup to be completed
                while (c.BackupInProgress) ;
            }
            catch (Exception)
            {
                MessageBox.Show("Could not create backup on controller.");
                return;
            }
            
            // Copy the remote backup directory from the controller to the local machine
            string localDir = Path.Combine(dirBackupPC, $"{c.IPAddress} {c.Name} {DateTime.Now.ToString("yyyy-MM-dd HH.mm.ss")}");
            try
            {
                c.FileSystem.GetDirectory(DIR_BACKUP_CTRL, localDir, false);
            }
            catch (Exception)
            {
                Directory.Delete(localDir, true);
                MessageBox.Show("Could not copy backup from controller.");
                return;
            }

            // Remove the remote backup directory from controller
            try
            {
                c.FileSystem.RemoveDirectory(DIR_BACKUP_CTRL, true);
            }
            catch (Exception)
            {
                MessageBox.Show("Could not remove temporary backup from controller.");
                return;
            }

            MessageBox.Show($"Successfully saved backup from controller to {localDir}");
        }

        /// <summary>
        /// Pick a local directory where the backups will be saved to.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonLocation_Click(object sender, EventArgs e)
        {
            using (var dialog = new FolderBrowserDialog())
            {
                DialogResult result = dialog.ShowDialog();

                if (result == DialogResult.OK && !string.IsNullOrWhiteSpace(dialog.SelectedPath))
                    SetPath(dialog.SelectedPath);
            }
        }


        /// <summary>
        /// Scan the network for controllers.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonScan_Click(object sender, EventArgs e)
        {
            NetworkScanner scanner = new NetworkScanner();
            scanner.Scan();
            System.Threading.Thread.Sleep(1000);
            scanner.Scan();

            foreach (ControllerInfo info in scanner.GetControllers())
            {
                ListViewItem item = new ListViewItem(new string[] { info.IPAddress.ToString(), info.Name, info.SystemName, info.Availability.ToString() }) { Tag = info };
                listView.Items.Add(item);
            }

            listView.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
        }

        /// <summary>
        /// Set the path and update the textbox.
        /// </summary>
        /// <param name="path"></param>
        private void SetPath(string path)
        {
            dirBackupPC = path;
            textBox.Text = path;
        }
    }
}