﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Windows.Forms;
using ABB.Robotics.Controllers;
using ABB.Robotics.Controllers.Discovery;

namespace ExampleDeviceManager
{
    /// <summary>
    /// Example application showing how the device tree of a controller can be accessed using the PCSDK.
    /// </summary>
    public partial class FormMain : Form
    {
        // The device nodes of the selected controller.
        private IEnumerable<IDeviceNode> deviceNodes;

        public FormMain()
        {
            InitializeComponent();
        }

        private void ListView_DoubleClick(object sender, EventArgs e)
        {
            SelectController();
        }

        private void ButtonSelect_Click(object sender, EventArgs e)
        {
            SelectController();
        }

        /// <summary>
        /// Attempt to access the device tree of the selected controller in the listView.
        /// </summary>
        private void SelectController()
        {
            if (listView.SelectedItems.Count > 0)
            {
                try
                {
                    ControllerInfo info = (ControllerInfo)(listView.SelectedItems[0]).Tag;
                    Controller controller = ControllerFactory.CreateFrom(info);

                    // Create a DeviceManager object that will access the device tree
                    DeviceManager deviceManager = new DeviceManager(controller);
                    deviceNodes = deviceManager.GetChildren();

                    // Remove the previous tree from the TreeView control
                    treeView.Nodes.Clear();
                    //treeView.Nodes.Add(GenerateTree(new TreeNode(info.SystemName), deviceNodes));
                }
                catch (Exception)
                {
                    MessageBox.Show($"Failed to access device tree.");
                }
            }
        }

        /// <summary>
        /// Scan the network for controllers.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonScan_Click(object sender, EventArgs e)
        {
            NetworkScanner scanner = new NetworkScanner();
            scanner.Scan();
            listView.Items.Clear();
            foreach (ControllerInfo info in scanner.GetControllers())
            {
                ListViewItem item = new ListViewItem(new string[] { info.IPAddress.ToString(), info.SystemName, info.ControllerName }) { Tag = info };
                listView.Items.Add(item);
            }
            listView.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
        }

        private void ButtonUpdate_Click(object sender, EventArgs e)
        {
            // All of the device tree is under the root node [0]
            UpdateTree(treeView.Nodes[0]);
        }

        /// <summary>
        /// Update the entire DeviceTree with new information from the DeviceNodes of the controller.
        /// </summary>
        /// <param name="parentNode"></param>
        private void UpdateTree(TreeNode parentNode)
        {
            if (parentNode.Tag != null)
                parentNode.Text = GetNodeText((IDeviceNode)parentNode.Tag);

            foreach (TreeNode childTree in parentNode.Nodes)
                UpdateTree(childTree);
        }

        /// <summary>
        /// Recursive function that will go through the entire TreeNode and update the values with information from the corresponding DeviceNode.
        /// It can be thought of as a conversion of the DeviceTree to the tree that the controller can use.
        /// </summary>
        /// <param name="parentNode"></param>
        /// <param name="childNodes"></param>
        /// <returns></returns>
        private TreeNode GenerateTree(TreeNode parentNode, IEnumerable<IDeviceNode> childNodes)
        {
            foreach (IDeviceNode node in childNodes)
            {
                // Generate a child tree for each child node in the device node.
                TreeNode childTree = new TreeNode(GetNodeText(node)) { Tag = node };
                // Recursively call this funciton to generate the grand children of the parent node.
                parentNode.Nodes.Add(GenerateTree(childTree, node.GetChildren()));
            }

            return parentNode;
        }

        /// <summary>
        /// Attempts to access the property value of a deviceNode and generate a text representation.
        /// If this fails it will simply return the node name.
        /// </summary>
        /// <param name="deviceNode"></param>
        /// <returns></returns>
        private static string GetNodeText(IDeviceNode deviceNode)
        {
            try
            {
                IPropertyNode property = (IPropertyNode)deviceNode;
                return $"{deviceNode.ToString()} {property.Value} {property.Unit}";
            }
            catch (Exception)
            {
                return $"{deviceNode.ToString()}";
            }
        }

        /// <summary>
        /// Read a specific node value given a path in the device tree.
        /// If the path does not lead to a PropertyNode it will return null.
        /// </summary>
        /// <param name="nodes"></param>
        /// <param name="path"></param>
        /// <returns></returns>
        public static IPropertyNode ReadDeviceNodes(IEnumerable<IDeviceNode> nodes, string[] path)
        {
            // Set the language to en-US and save the old language
            CultureInfo oldCulture = CultureInfo.CurrentCulture;
            CultureInfo.CurrentUICulture = CultureInfo.CreateSpecificCulture("en-US");

            if (path.Length < 1)
                return null;

            IDeviceNode node = nodes.FirstOrDefault(e => e.ToString().Equals(path[0]));

            if (node == null)
                return null;

            for (int i = 1; i < path.Length; i++)
                if ((node = node.GetChildren().FirstOrDefault(x => x.ToString().Equals(path[i]))) == null)
                    return null;

            // Reset the language to the old language
            CultureInfo.CurrentUICulture = oldCulture;

            return (IPropertyNode)node;
        }

        /// <summary>
        /// Attempts to read a property node given the path in the text box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonGetValue_Click(object sender, EventArgs e)
        {
            if (deviceNodes != null)
            {
                IPropertyNode propertyNode = ReadDeviceNodes(deviceNodes, textBoxPath.Text.Split('/'));
                if (propertyNode != null)
                    textBoxValue.Text = propertyNode.Value;
            }
        }
    }
}
