﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using ABB.Robotics.Controllers;
using ABB.Robotics.Controllers.Discovery;

namespace ExampleGuidScan
{
    /// <summary>
    /// An example application showing how Guids can be used to connect to controllers.
    /// The Guids can also be saved and loaded at a later time to connect to specific controllers.
    /// </summary>
    public partial class FormMain : Form
    {
        public FormMain()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Scan the network for available controllers and populate the listView.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonScan_Click(object sender, EventArgs e)
        {
            NetworkScanner scanner = new NetworkScanner();
            scanner.Scan();
            UpdateListView(scanner.GetControllers());
        }

        /// <summary>
        /// Opens a save dialog to save the selected controllers Guids to a file.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog
            {
                Filter = "Text File|*.txt",
                Title = "Save Guids"
            };

            if (dialog.ShowDialog() == DialogResult.OK && dialog.FileName != "")
            {
                int n = listView.SelectedItems.Count;

                string[] selectedGuids = new string[n];
                for (int i = 0; i < n; i++)
                    selectedGuids[i] = ((ControllerInfo)listView.SelectedItems[i].Tag).SystemId.ToString();
                
                File.WriteAllLines(dialog.FileName, selectedGuids);
            }
        }

        /// <summary>
        /// Clears the list of discovered controllers.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonClear_Click(object sender, EventArgs e)
        {
            listView.Items.Clear();
            // Call to update the number of controllers selected label
            ListView_ItemSelectionChanged(null, null);
        }

        /// <summary>
        /// Loads controllers from a previously saved .txt file containing the Guids of the controllers.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonLoad_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog
            {
                Filter = "Text File|*.txt",
                Title = "Load Guids"
            };

            if (dialog.ShowDialog() == DialogResult.OK && dialog.FileName != "")
            {
                try
                {
                    Guid[] guids = File.ReadAllLines(dialog.FileName).Select(g => new Guid(g)).ToArray();
                    NetworkScanner networkScanner = new NetworkScanner();
                    List<ControllerInfo> controllerInfoList = new List<ControllerInfo>();

                    for (int i = 0; i < guids.Length; i++)
                        if (networkScanner.TryFind(guids[i], 1000, 2, out ControllerInfo info))
                            controllerInfoList.Add(info);

                    UpdateListView(controllerInfoList.ToArray());
                }
                catch (Exception)
                {
                    // Handle the exception
                }
            }
        }

        /// <summary>
        /// Populates the listView with controllers
        /// </summary>
        /// <param name="controllers"></param>
        private void UpdateListView(ControllerInfo[] controllers)
        {
            listView.Items.Clear();
            foreach (ControllerInfo info in controllers)
            {
                ListViewItem item = new ListViewItem(new string[] { info.IPAddress.ToString(), info.SystemId.ToString(), info.Name, info.SystemName }) { Tag = info };
                listView.Items.Add(item);
            }
            listView.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
        }

        /// <summary>
        /// Called upon when the number of selected controllers change.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListView_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            labelCount.Text = $"Controllers marked: {listView.SelectedItems.Count}";
        }
    }
}