﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using ABB.Robotics.Controllers;
using ABB.Robotics.Controllers.Discovery;

namespace ExampleNetworkWatcher
{
    /// <summary>
    /// Example application showing how the NetworkWatcher object can be used.
    /// It is important to remember that the NetworkWatcher will only detect CHANGES to the list of controllers
    /// made available by the process RobNetScanHost.exe. This means that it the process already have been running
    /// for a while and discovered controllers, the NetworkWatcher will not detect these when this application starts.
    /// </summary>
    public partial class FormMain : Form
    {
        private List<ControllerInfo> listedControllers = new List<ControllerInfo>();
        private NetworkScanner networkScanner;
        private NetworkWatcher networkWatcher;
        // Boolean indicating whether the NetworkWatcher is running.
        private bool running = false;

        public FormMain()
        {
            InitializeComponent();
        }

        private void ButtonToggle_Click(object sender, EventArgs e)
        {
            // Toggle the NetworkWatcher.
            if (running)
            {
                running = false;
                buttonToggle.Text = "Start NetworkScanner";
                networkWatcher.Dispose();
            }
            else
            {
                running = true;
                buttonToggle.Text = "Stop NetworkScanner";

                // Use the NetworkScanner to check for already discovered controllers on the network.
                networkScanner = new NetworkScanner();
                networkScanner.Scan();

                // Add the controllers that are already discovered on the network to the listed controllers.
                foreach (ControllerInfo info in networkScanner.GetControllers())
                {
                    // If the controller is not already in our list, add it.
                    if (!listedControllers.Contains(info))
                    {
                        listedControllers.Add(info);
                        Log($"Added controller already discovered by RobNetScanHost.exe: {info.IPAddress}");
                    }
                }

                // Create the NetworkWatcher object and tie it to events when controllers are lost and discovered.
                networkWatcher = new NetworkWatcher(networkScanner.GetControllers(), true);
                networkWatcher.Found += NetworkWatcher_Found;
                networkWatcher.Lost += NetworkWatcher_Lost;

                UpdateListView();
            }
        }

        /// <summary>
        /// Event called upon when RobNetScanHost.exe discovers a new controller.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NetworkWatcher_Lost(object sender, NetworkWatcherEventArgs e)
        {
            Log($"RobNetScanHost.exe lost controller {e.Controller.IPAddress}");
            listedControllers.Remove(e.Controller);
            UpdateListView();
        }

        /// <summary>
        /// Event called upon when RobNetScanHost.exe loses a controller.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NetworkWatcher_Found(object sender, NetworkWatcherEventArgs e)
        {
            Log($"RobNetScanHost.exe found controller {e.Controller.IPAddress}");
            listedControllers.Add(e.Controller);
            UpdateListView();
        }

        /// <summary>
        /// Clears the listed controllers in the application.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonForget_Click(object sender, EventArgs e)
        {
            listedControllers.Clear();
            UpdateListView();
        }

        /// <summary>
        /// Populates the ListView with the discovered controllers.
        /// </summary>
        private void UpdateListView()
        {
            listView.Items.Clear();
            foreach (ControllerInfo info in listedControllers)
                listView.Items.Add(new ListViewItem(new string[] { info.IPAddress.ToString(), info.Name }));
            listView.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
        }

        /// <summary>
        /// Logs a message to the TextBox with a timestamp added.
        /// </summary>
        /// <param name="msg"></param>
        private void Log(string msg)
        {
            textBox.AppendText($"{DateTime.Now.ToString("HH:mm:ss")}: {msg}\n");
        }

        /// <summary>
        /// Clears the TextBox log.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonClear_Click(object sender, EventArgs e)
        {
            textBox.Clear();
        }
    }
}